
from gen3.indexclient.client import IndexClient


def test_index_list():
    indexHost = "https://nci-crdc-demo.datacommons.io/index/"
    indexVersion = ("v0",)
    indexAuth = ()

    ic = IndexClient(indexHost, indexVersion, indexAuth)
    doc = ic.get("001cf43a-40ed-4c45-8da4-0427d7a750dd")
    assert doc



import json
import requests


class Gen3SubmissionQueryError(Exception):
    pass


class Gen3Submission:
    """Submit/Export/Query data from a Gen3 Submission system.

    A class for interacting with the Gen3 submission services.
    Supports submitting and exporting from Sheepdog.
    Supports GraphQL queries through Peregrine.

    Args:
        endpoint (str): The URL of the data commons.
        auth_provider (Gen3Auth): A Gen3Auth class instance.

    Examples:
        This generates the Gen3Submission class pointed at the sandbox commons while
        using the credentials.json downloaded from the commons profile page.

        >>> endpoint = "https://nci-crdc-demo.datacommons.io"
        ... auth = Gen3Auth(endpoint, refresh_file="credentials.json")
        ... sub = Gen3Submission(endpoint, auth)

    """

    def __init__(self, endpoint, auth_provider):
        self._auth_provider = auth_provider
        self._endpoint = endpoint

    def query(self, query_txt, variables=None, max_tries=1):
        """Execute a GraphQL query against a data commons.

        Args:
            query_txt (str): Query text.
            variables (:obj:`object`, optional): Dictionary of variables to pass with the query.
            max_tries (:obj:`int`, optional): Number of times to retry if the request fails.
   
        Examples:
            This executes a query to get the list of all the project codes for all the projects
            in the data commons.

            >>> query = "{ project(first:0) { code } }"
            ... Gen3Submission.query(query)

        """
        api_url = "{}/api/v0/submission/graphql".format(self._endpoint)
        if variables == None:
            query = {"query": query_txt}
        else:
            query = {"query": query_txt, "variables": variables}

        tries = 0
        while tries < max_tries:
            output = requests.post(api_url, auth=self._auth_provider, json=query).text
            data = json.loads(output)

            if "errors" in data:
                raise Gen3SubmissionQueryError(data["errors"])

            if not "data" in data:
                print(query_txt)
                print(data)

            tries += 1

        return data

    def export_node(self, program, project, uuid):
        """Export a single node into json.

        Args:
            program (str): The program the node is under.
            project (str): The project the node is under.
            uuid (str): The UUID of the node to export.
   
        Examples:
            This exports a node from the sandbox commons.

            >>> Gen3Submission.export_node("DCF", "CCLE", "d70b41b9-6f90-4714-8420-e043ab8b77b9")

        """
        api_url = "{}/api/v0/submission/{}/{}/export?ids={}&format=json".format(
            self._endpoint, program, project, uuid
        )
        output = requests.get(api_url, auth=self._auth_provider).text
        data = json.loads(output)
        return data

    def export_node_all_type(self, program, project, node_type):
        """Export all nodes of a single type.

        Args:
            program (str): The program to export nodes under.
            project (str): The project to export nodes under.
            node_type (str): The label of the node type to export.
   
        Examples:
            This exports all "sample" types from the CCLE project on the sandbox commons.

            >>> Gen3Submission.export_node_all_type("DCF", "CCLE", "sample")

        """
        api_url = "{}/api/v0/submission/{}/{}/export/?node_label={}&format=json".format(
            self._endpoint, program, project, node_type
        )
        output = requests.get(api_url, auth=self._auth_provider).text
        data = json.loads(output)
        return data

    def submit_node(self, program, project, json):
        """Submit a node to a project.

        Args:
            program (str): The program to submit to.
            project (str): The project to submit to.
            json (object): The json of the node to submit
   
        Examples:
            This submits a node to the CCLE project on the sandbox commons.

            >>> Gen3Submission.submit_node("DCF", "CCLE", json)

        """
        api_url = "{}/api/v0/submission/{}/{}".format(self._endpoint, program, project)
        output = requests.put(api_url, auth=self._auth_provider, json=json).text
        return output

    def delete_project(self, program, project):
        """Delete a project.

        This deletes an empty project from the commons.

        Args:
            program (str): The program containing the project to delete.
            project (str): The project to delete.
   
        Examples:
            This deletes the "CCLE" project from the "DCF" program.

            >>> Gen3Submission.delete_project("DCF", "CCLE")

        """
        api_url = "{}/api/v0/submission/{}/{}".format(self._endpoint, program, project)
        output = requests.delete(api_url, auth=self._auth_provider).text
        return output

    def delete_program(self, program):
        """Delete a program.

        This deletes an empty program from the commons.

        Args:
            program (str): The program to delete.
   
        Examples:
            This deletes the "DCF" program.

            >>> Gen3Submission.delete_program("DCF")

        """
        api_url = "{}/api/v0/submission/{}".format(self._endpoint, program)
        output = requests.delete(api_url, auth=self._auth_provider).text
        return output

    def get_dictionary_all(self):
        """Returns the dictionary object for a commons.

        This gets the current json dictionary schema for a commons.
   
        Examples:
            This returns the dictionary schema for a commons.

            >>> Gen3Submission.get_dictionary_all()

        """
        return get_dictionary_node("_all")

    def get_dictionary_node(self, node_type):
        """Returns the dictionary schema for a specific node.

        This gets the current json dictionary schema for a specific node type in a commons.

        Args:
            node_type (str): The node_type to retrieve.
   
        Examples:
            This returns the dictionary schema the "subject" node.

            >>> Gen3Submission.get_dictionary_node("subject")

        """
        api_url = "{}/api/v0/submission/_dictionary/{}".format(
            self._endpoint, node_type
        )
        output = requests.get(api_url).text
        data = json.loads(output)
        return data

    def get_graphql_schema(self):
        """Returns the GraphQL schema for a commons.

        This runs the GraphQL introspection query against a commons and returns the results.
   
        Examples:
            This returns the GraphQL schema.

            >>> Gen3Submission.get_graphql_schema()

        """
        api_url = "{}/api/v0/submission/getschema".format(self._endpoint)
        output = requests.get(api_url).text
        data = json.loads(output)
        return data



import json
import requests


class Gen3FileError(Exception):
    pass


class Gen3File:
    """For interacting with Gen3 file management features.

    A class for interacting with the Gen3 file download services.
    Supports getting presigned urls right now.

    Args:
        endpoint (str): The URL of the data commons.
        auth_provider (Gen3Auth): A Gen3Auth class instance.

    Examples:
        This generates the Gen3File class pointed at the sandbox commons while
        using the credentials.json downloaded from the commons profile page.

        >>> endpoint = "https://nci-crdc-demo.datacommons.io"
        ... auth = Gen3Auth(endpoint, refresh_file="credentials.json")
        ... sub = Gen3File(endpoint, auth)

    """

    def __init__(self, endpoint, auth_provider):
        self._auth_provider = auth_provider
        self._endpoint = endpoint

    def get_presigned_url(self, guid, protocol="http"):
        """Generates a presigned URL for a file.

        Retrieves a presigned url for a file giving access to a file for a limited time.

        Args:
            guid (str): The GUID for the object to retrieve.
            protocol (:obj:`str`, optional): The protocol to use for picking the available URL for generating the presigned URL.

        Examples:

            >>> Gen3File.get_presigned_url(query)

        """
        api_url = "{}/user/data/download/{}&protocol={}".format(
            self._endpoint, guid, protocol
        )
        output = requests.get(api_url, auth=self._auth_provider).text
        data = json.loads(output)
        return data



import json
from requests.auth import AuthBase
import requests


class Gen3AuthError(Exception):
    pass


class Gen3Auth(AuthBase):
    """Gen3 auth helper class for use with requests auth.

    Implements requests.auth.AuthBase in order to support JWT authentication.
    Generates access tokens from the provided refresh token file or string.
    Automatically refreshes access tokens when they expire.

    Args:
        endpoint (str): The URL of the data commons.
        refresh_file (str): The file containing the downloaded json web token.
        refresh_token (str): The json web token.

    Examples:
        This generates the Gen3Auth class pointed at the sandbox commons while
        using the credentials.json downloaded from the commons profile page.

        >>> auth = Gen3Auth("https://nci-crdc-demo.datacommons.io", refresh_file="credentials.json")

    """

    def __init__(self, endpoint, refresh_file=None, refresh_token=None):
        if not refresh_file and not refresh_token:
            raise ValueError(
                "Either parameter 'refresh_file' or parameter 'refresh_token' must be specified."
            )

        if refresh_file and refresh_token:
            raise ValueError(
                "Only one of 'refresh_file' and 'refresh_token' must be specified."
            )

        self._refresh_file = refresh_file
        self._refresh_token = refresh_token

        if refresh_file:
            try:
                file_data = open(self._refresh_file).read()
                self._refresh_token = json.loads(file_data)
            except Exception as e:
                raise ValueError(
                    "Couldn't load your refresh token file: {}\n{}".format(
                        self._refresh_file, str(e)
                    )
                )

        self._access_token = None
        self._endpoint = endpoint

    def __call__(self, request):
        """Adds authorization header to the request

        This gets called by the python.requests package on outbound requests
        so that authentication can be added.

        Args:
            request (object): The incoming request object

        """
        request.headers["Authorization"] = self._get_auth_value()
        request.register_hook("response", self._handle_401)
        return request

    def _handle_401(self, response, **kwargs):
        """Handles failed requests when authorization failed.

        This gets called after a failed request when an HTTP 401 error
        occurs. This then tries to refresh the access token in the event
        that it expired.

        Args:
            request (object): The failed request object

        """
        if not response.status_code == 401 and not response.status_code == 403:
            return response

        # Free the original connection
        response.content
        response.close()

        # copy the request to resend
        newreq = response.request.copy()

        self._access_token = None
        newreq.headers["Authorization"] = self._get_auth_value()

        _response = response.connection.send(newreq, **kwargs)
        _response.history.append(response)
        _response.request = newreq

        return _response

    def _get_auth_value(self):
        """Returns the Authorization header value for the request

        This gets called when added the Authorization header to the request.
        This fetches the access token from the refresh token if the access token is missing.

        """
        if not self._access_token:
            auth_url = "{}/user/credentials/cdis/access_token".format(self._endpoint)
            try:
                self._access_token = requests.post(
                    auth_url, json=self._refresh_token
                ).json()["access_token"]
            except Exception as e:
                raise Gen3AuthError(
                    "Failed to authenticate to {}\n{}".format(auth_url, str(e))
                )

        return "Bearer " + self._access_token



from setuptools import setup, find_packages
from subprocess import check_output


def get_version():
    try:
        tag = check_output(
            ["git", "describe", "--tags", "--abbrev=0", "--match=[0-9]*"]
        )
        return tag.decode('utf-8').strip("\n")
    except Exception:
        # if somehow you get the repo not from git,
        # hardcode default major version
        return "0.1.0"


with open("README.md", "r") as f:
    README = f.read()

with open("CHANGELOG.md", "r") as f:
    CHANGELOG = f.read()

setup(
    name="gen3",
    description=(
        "The Gen3 SDK makes it easy to utilize " "functionality in Gen3 data commons. "
    ),
    long_description=README + "\n\n" + CHANGELOG,
    long_description_content_type="text/markdown",
    author="Center for Translational Data Science",
    author_email="support@datacommons.io",
    license="Apache 2.0",
    url="https://gen3.org/",
    version=get_version(),
    packages=find_packages(),
    install_requires=["requests"],
    package_data={"": ["LICENSE"]},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Topic :: Scientific/Engineering",
    ],
)



def test_import():

    import gen3


def test_indexclient_import():

    from gen3.indexclient.client import IndexClient


def test_auth_import():

    from gen3.auth import Gen3Auth


def test_submission_import():

    from gen3.submission import Gen3Submission


def test_file_import():

    from gen3.file import Gen3File
